|***rev.*:**|0.1.30|
|:---|---:|
|***date***:|2025-03-29|

## Introduction

This paper contains descriptions of all classes presented in "`baseClass.js`".

## Module classes

<a name="TItemsList"></a>
### **TItemsList**

This class implements a base container for managing an objects such as lists.

#### class constructor

The class constructor creates a new instance of the class.

##### constructor parameters

The class constructor receives no arguments.

#### class properties

<a name="TItemsList+count"></a>
##### **count**

| property type | read only | description |
|---|---|:---|
| `number` | yes | contains a quantity of the elements |

<a name="TItemsList+curIndex"></a>
##### **curIndex**

| property type | read only | description |
|---|---|:---|
| `number` | no | represents an index of the current element |

> NOTE: see the class [`setIndex`](#TItemsList+setIndex) method for more details on acceptable values which can be assigned to the property.

<a name="TItemsList+curItem"></a>
##### **curItem**

| property type | read only | description |
|---|---|:---|
| `any` | no | represents an element addressed by the `curIndex` |

> NOTE: see the class [`getItem`](#TItemsList+getItem) and [`setItem`](#TItemsList+setItem) methods for more details.

<a name="TItemsList+minIndex"></a>
##### **minIndex**

| property type | read only | description |
|---|---|:---|
| `number` | yes | represents a minimum possible index for an element |

> NOTE: If an instance contains no elements the `-1` returned.

<a name="TItemsList+maxIndex"></a>
##### **maxIndex**

| property type | read only | description |
|---|---|:---|
| `number` | yes | represents a maximum possible index for an element |

> NOTE: If an instance contains no elements the `-1` returned.

#### class methods

<a name="TItemsList+clear"></a>
##### **clear()** => `void`

This method deletes all items from the list.

<a name="TItemsList+isEmpty"></a>
##### **isEmpty()** => `boolean`

This method returns `true` if no items exists in the list.

<a name="TItemsList+isNotEmpty"></a>
##### **isNotEmpty()** => `boolean`

This method returns `true` if at least one item is exists in the list.

<a name="TItemsList+chkIndex"></a>
##### **chkIndex(value)** => `boolean`

This method checks if the given `value` represents a valid index value and fits the range of all possible indexes belonging to the list instance. If so `true` is returned.

| parameter name | value type | default value | description |
|:---|---|---:|:---|
| `value` | `number` or `string` | --- | a value to be verified |

<a name="TItemsList+setIndex"></a>
##### **setIndex(value)** => `boolean`

This method sets a current item index for the list if the given `value` represents a valid index value and fits the range of all possible indexes belonging to the list. If succeed `true` is returned.

| parameter name | value type | default value | description |
|:---|---|---:|:---|
| `value` | `number` or `string` | --- | a value of an index to be set |

> \[!] NOTE: You can\'t use a value of `-1` (*a special index value*) by this method. If you want to reset the value of a `curIndex` property to its initial status use the class [`rstIndex`](#TItemsList+rstIndex) method.

<a name="TItemsList+rstIndex"></a>
##### **rstIndex()** => `void`

This method resets a current item index for the list.

> Note: You need to use this method if you want to reset the value of a `curIndex` property to its initial status which is set to `-1`. You can\'t perform for that a direct action on a `curIndex` property or with the `setIndex` method.

<a name="TItemsList+srchIndex"></a>
##### **srchIndex(item\[, index])** => `number`

This method seeks an index of a given `item` and if it is found returns its index in the list. If failed `-1` is returned.

| parameter name | value type | default value | description |
|:---|---|---:|:---|
| `item` | `any` | --- | some element |
| `index` | `number` or `string` | --- | a value of an index to start |

Searching starts from the position given by the `index`, if it is given and is valid. By default the searching starts from the first item in the list.

> Note: In case a given `index` is valid but its value is greater than the max index of the list, the method will fail.

<a name="TItemsList+getItem"></a>
##### **getItem(index)** => `any`

This method returns the item addressed by the `index`.

| parameter name | value type | default value | description |
|:---|---|---:|:---|
| `index` | `number` or `string` | --- | an index for the target element |

> Note: If failed `undefined` is returned.

<a name="TItemsList+setItem"></a>
##### **setItem(index, item)** => `boolean`

This method places the given `item` into position addressed by the `index`. If succeed `true` is returned.

| parameter name | value type | default value | description |
|:---|---|---:|:---|
| `index` | `number` or `string` | --- | an index for the target element |
| `item` | `any` | --- | some element |

> Note: You can\'t apply the item, which value is `undefined`, to the list members. In such a case the method will failed.

<a name="TItemsList+addItem"></a>
##### **addItem(item)** => `boolean`

This method adds the given `item` to a list members. If succeed `true` is returned.

| parameter name | value type | default value | description |
|:---|---|---:|:---|
| `item` | `any` | --- | some element |

> Note: You can\'t add the item, which value is `undefined`, to the list members. In such a case the method will failed.

<a name="TItemsList+addItemEx"></a>
##### **addItemEx(item\[, options])** => `number`

This method adds the given `item` to a list members. If succeed the item index will be returned, if else `-1` is returned.

| parameter name | value type | default value | description |
|:---|---|---:|:---|
| `item` | `any` | --- | some element |
| `options` | `boolean` | `false` | a flag that indicates whether to correct a `curIndex` value |

If `options` parameter is given and is set to `true`, in case of success, the `curIndex` property is adjusted by the method.

<a name="TItemsList+delItem"></a>
##### **delItem(index\[, options])** => `boolean`

This method deletes the item addressed by the given `index` from a list members. If succeed `true` is returned.

| parameter name | value type | default value | description |
|:---|---|---:|:---|
| `index` | `number` or `string` | --- | an index for the target element |
| `options` | `boolean` | `true` | a flag that indicates whether to correct a `curIndex` value |

If `options` parameter is set to `true` (*a default value*), the `curIndex` property is adjusted by the method. If it is set to `false` the `curIndex` property is not adjusted before a value of the property exited out of the list indexes range that caused the property to be reset.

<a name="TItemsList+delItemEx"></a>
##### **delItemEx(index\[, options])** => `any`

This method deletes the item addressed by the given `index` from a list members. If succeed, the method returns an item that was deleted, if failed, `undefined` is returned.

| parameter name | value type | default value | description |
|:---|---|---:|:---|
| `index` | `number` or `string` | --- | an index for the target element |
| `options` | `boolean` | `true` | a flag that indicates whether to correct a `curIndex` value |

The behavior of this method is similar to the class `delItem` method except a type of the result.

<a name="TItemsList+loadItems"></a>
##### **loadItems(list, options)** => `number`

This method adds all given items listed by the `list` to a list members and returns a quantity of the items that was added successfully.

| parameter name | value type | default value | description |
|:---|---|---:|:---|
| `list` | `any` | --- | a list of some elements to load |
| `options` | `boolean`, `object` | --- | an options |

> \[!] NOTE: `[since v0.2.2]` a use of a `boolean` type as a value of an `options` are deprecated and will be dropped soon.

The `options` parameter is an `object` that contains the following set of parameters:

| option name | value type | default value | description |
|:---|---|---:|:---|
| `useClear` | `boolean` | `true` | if set to `true`, before the load, the list is cleared |
| `curIndex` | `number` or `string` | --- | if given, after the load, defines an index which will be set as current |
| `resetIndex` | `boolean` | `false` | if set to `true`, after the load, the list index will be reset |

> ***Note**: The `resetIndex` is used only if `useClear` is `false` and an attempt to set `curIndex` is failed.*

#### class methods (*special*)

<a name="TItemsList+forEach"></a>
##### **forEach(cb)** => `void`

> since: \[v0.2.1]

This method calls a given function for each item in a list.

| parameter name | value type | default value | description |
|:---|---|---:|:---|
| `cb` | `function` | --- | a user defined function to process element |

<a name="TItemsListEx"></a>
### **TItemsListEx**

This class extends a base container provided by the [`TItemsList`](#TItemsList) class with a new functionalities.

#### class constructor

The class constructor creates a new instance of the class.

##### constructor parameters

The class constructor receives no arguments.

#### class properties

<a name="TItemsListEx+count"></a>
##### **count**

| property type | read only | description |
|---|---|:---|
| `number` | yes | contains a quantity of the elements |

<a name="TItemsListEx+curIndex"></a>
##### **curIndex**

| property type | read only | description |
|---|---|:---|
| `number` | no | represents an index of the current element |

> NOTE: see the class [`setIndex`](#TItemsListEx+setIndex) method for more details on acceptable values which can be assigned to the property.

<a name="TItemsListEx+curItem"></a>
##### **curItem**

| property type | read only | description |
|---|---|:---|
| `any` | no | represents an element addressed by the `curIndex` |

> NOTE: see the class [`getItem`](#TItemsListEx+getItem) and [`setItem`](#TItemsListEx+setItem) methods for more details.

<a name="TItemsListEx+minIndex"></a>
##### **minIndex**

| property type | read only | description |
|---|---|:---|
| `number` | yes | represents a minimum possible index for an element |

> NOTE: If an instance contains no elements the `-1` returned.

<a name="TItemsListEx+maxIndex"></a>
##### **maxIndex**

| property type | read only | description |
|---|---|:---|
| `number` | yes | represents a maximum possible index for an element |

> NOTE: If an instance contains no elements the `-1` returned.

<a name="TItemsListEx+prevIndex"></a>
##### **prevIndex**

| property type | read only | description |
|---|---|:---|
| `number` | yes | represents an index of an item that is previous to the current element |

> NOTE: If an instance contains no elements the `-1` returned.

<a name="TItemsListEx+nextIndex"></a>
##### **nextIndex**

| property type | read only | description |
|---|---|:---|
| `number` | yes | represents an index of an item that is next to the current element |

> NOTE: If an instance contains no elements the `-1` returned.

#### class methods

<a name="TItemsListEx+clear"></a>
##### **clear()** => `void`

This method deletes all items from the list.

<a name="TItemsListEx+isEmpty"></a>
##### **isEmpty()** => `boolean`

This method returns `true` if no items exists in the list.

<a name="TItemsListEx+isNotEmpty"></a>
##### **isNotEmpty()** => `boolean`

This method returns `true` if at least one item is exists in the list.

<a name="TItemsListEx+chkIndex"></a>
##### **chkIndex(value)** => `boolean`

This method checks if the given `value` represents a valid index value and fits the range of all possible indexes belonging to the list instance. If so `true` is returned.

| parameter name | value type | default value | description |
|:---|---|---:|:---|
| `value` | `number` or `string` | --- | a value to be verified |

<a name="TItemsListEx+chkIndexEx"></a>
##### **chkIndexEx(value\[, mode])** => `boolean`\|`number`

This method is similar to a `chkIndex` methods but receives a `mode` parameter which allow to choose what type of result the function returned.

| parameter name | value type | default value | description |
|:---|---|---:|:---|
| `value` | `number` or `string` | --- | a value to be verified |
| `value` | `boolean` | `false` | a flag to switch a mode |

The `mode` parameter treats as follows:

- if set to `false`, the result returned by the method is a `boolean` value;
- if `mode` parameter set to `true`, the result returned by the method has an index value.

<a name="TItemsListEx+setIndex"></a>
##### **setIndex(value)** => `boolean`

This method sets a current item index for the list if the given `value` represents a valid index value and fits the range of all possible indexes belonging to the list. If succeed `true` is returned.

| parameter name | value type | default value | description |
|:---|---|---:|:---|
| `value` | `number` or `string` | --- | a value of an index to be set |

> \[!] NOTE: You can\'t use a value of `-1` (*a special index value*) by this method. If you want to reset the value of a `curIndex` property to its initial status use the class [`rstIndex`](#TItemsListEx+rstIndex) method.

<a name="TItemsListEx+rstIndex"></a>
##### **rstIndex()** => `void`

This method resets a current item index for the list.

> Note: You need to use this method if you want to reset the value of a `curIndex` property to its initial status which is set to `-1`. You can\'t perform for that a direct action on a `curIndex` property or with the `setIndex` method.

<a name="TItemsListEx+srchIndex"></a>
##### **srchIndex(item\[, index])** => `number`

This method seeks an index of a given `item` and if it is found returns its index in the list. If failed `-1` is returned.

| parameter name | value type | default value | description |
|:---|---|---:|:---|
| `item` | `any` | --- | some element |
| `index` | `number` or `string` | --- | a value of an index to start |

Searching starts from the position given by the `index`, if it is given and is valid. By default the searching starts from the first item in the list.

> Note: In case a given `index` is valid but its value is greater than the max index of the list, the method will fail.

<a name="TItemsListEx+srchNextIndex"></a>
##### **srchNextIndex(item)** => `number`

This method is similar to a `srchIndex` method but it starts from where the previous search calls was stopped.

| parameter name | value type | default value | description |
|:---|---|---:|:---|
| `item` | `any` | --- | some element |

> \[!] NOTE: You can\'t use this method if a previous search operation has failed. In such a case, you must start a new search with a `srchIndex` method. Also the method will failed if a `srchIndex` method was not called early.

<a name="TItemsListEx+getItem"></a>
##### **getItem(index)** => `any`

This method returns the item addressed by the `index`.

| parameter name | value type | default value | description |
|:---|---|---:|:---|
| `index` | `number` or `string` | --- | an index for the target element |

> Note: If failed `undefined` is returned.

<a name="TItemsListEx+setItem"></a>
##### **setItem(index, item)** => `boolean`

This method places the given `item` into position addressed by the `index`. If succeed `true` is returned.

| parameter name | value type | default value | description |
|:---|---|---:|:---|
| `index` | `number` or `string` | --- | an index for the target element |
| `item` | `any` | --- | some element |

> Note: You can\'t apply the item, which value is `undefined`, to the list members. In such a case the method will failed.

<a name="TItemsListEx+addItem"></a>
##### **addItem(item)** => `boolean`

This method adds the given `item` to a list members. If succeed `true` is returned.

| parameter name | value type | default value | description |
|:---|---|---:|:---|
| `item` | `any` | --- | some element |

> Note: You can\'t add the item, which value is `undefined`, to the list members. In such a case the method will failed.

<a name="TItemsListEx+addItemEx"></a>
##### **addItemEx(item\[, options])** => `number`

This method adds the given `item` to a list members. If succeed the item index will be returned, if else `-1` is returned.

| parameter name | value type | default value | description |
|:---|---|---:|:---|
| `item` | `any` | --- | some element |
| `options` | `boolean` | `false` | a flag that indicates whether to correct a `curIndex` value |

If `options` parameter is given and is set to `true`, in case of success, the `curIndex` property is adjusted by the method.

<a name="TItemsListEx+delItem"></a>
##### **delItem(index\[, options])** => `boolean`

This method deletes the item addressed by the given `index` from a list members. If succeed `true` is returned.

| parameter name | value type | default value | description |
|:---|---|---:|:---|
| `index` | `number` or `string` | --- | an index for the target element |
| `options` | `boolean` | `true` | a flag that indicates whether to correct a `curIndex` value |

If `options` parameter is set to `true` (*a default value*), the `curIndex` property is adjusted by the method. If it is set to `false` the `curIndex` property is not adjusted before a value of the property exited out of the list indexes range that caused the property to be reset.

<a name="TItemsListEx+delItemEx"></a>
##### **delItemEx(index\[, options])** => `any`

This method deletes the item addressed by the given `index` from a list members. If succeed, the method returns an item that was deleted, if failed, `undefined` is returned.

| parameter name | value type | default value | description |
|:---|---|---:|:---|
| `index` | `number` or `string` | --- | an index for the target element |
| `options` | `boolean` | `true` | a flag that indicates whether to correct a `curIndex` value |

The behavior of this method is similar to the class `delItem` method except a type of the result.

<a name="TItemsListEx+loadItems"></a>
##### **loadItems(list, options)** => `number`

This method adds all given items listed by the `list` to a list members and returns a quantity of the items that was added successfully.

| parameter name | value type | default value | description |
|:---|---|---:|:---|
| `list` | `any` | --- | a list of some elements to load |
| `options` | `boolean`, `object` | --- | an options |

> \[!] NOTE: `[since v0.2.2]` a use of a `boolean` type as a value of an `options` are deprecated and will be dropped soon.

The `options` parameter is an `object` that contains the following set of parameters:

| option name | value type | default value | description |
|:---|---|---:|:---|
| `useClear` | `boolean` | `true` | if set to `true`, before the load, the list is cleared |
| `curIndex` | `number` or `string` | --- | if given, after the load, defines an index which will be set as current |
| `resetIndex` | `boolean` | `false` | if set to `true`, after the load, the list index will be reset |

> ***Note**: The `resetIndex` is used only if `useClear` is `false` and an attempt to set `curIndex` is failed.*

#### class methods (*special*)

<a name="TItemsListEx+forEach"></a>
##### **forEach(cb)** => `void`

> since: \[v0.2.1]

This method calls a given function for each item in a list.

| parameter name | value type | default value | description |
|:---|---|---:|:---|
| `cb` | `function` | --- | a user defined function to process element |

<a name="TNamedItemsCollection"></a>
### **TNamedItemsCollection**

This class implements a simple collection of a named elements.

#### class constructor

The class constructor creates a new instance of the class.

##### constructor parameters

The class receives an `object` as its argument which contains an options listed below:

| option name | option type | default value | description |
|:---|---|---:|:---|
| `forceIfKeyExists` | `boolean` | `false` | alters a behavior of the `add` and `rename` class methods. If set to `true`, the replacement of existing keys is allowed |
| `userFn` | `object` | --- | a set of a user defined functions |

The `userFn` provides the set of the next parameters:

| option name | option type | default value | description |
|:---|---|---:|:---|
| `checkIndexFn` | `function` | --- | defines a function that checks whether or not the given key can be accepted |
| `readIndexFn` | `function` | --- | defines a function that handles the given key |

The `checkIndexFn` function gets the key that must be verified and returns a `boolean` value as a result of a verification process.

The `readIndexFn` function gets a key and returns it transformed according to the rules set by some user logics.

#### class properties

<a name="TNamedItemsCollection+keys"></a>
##### **keys**

| property type | read only | description |
|---|---|:---|
| `any[]` | yes | returns an array of all collection keys |

<a name="TNamedItemsCollection+values"></a>
##### **values**

| property type | read only | description |
|---|---|:---|
| `any[]` | yes | returns an array of all collection values |

<a name="TNamedItemsCollection+entries"></a>
##### **entries**

| property type | read only | description |
|---|---|:---|
| `any[][]` | yes | returns an array of all collection entries as a "key-value" pairs |

<a name="TNamedItemsCollection+size"></a>
##### **size**

| property type | read only | description |
|---|---|:---|
| `number` | yes | returns a collection size |

#### class methods

<a name="TNamedItemsCollection+clear"></a>
##### **clear()** => `void`

This method deletes all items that a collection holds.

<a name="TNamedItemsCollection+has"></a>
##### **has(key)** => `boolean`

This method checks if the given `key` exists in the collection.

| parameter name | value type | default value | description |
|:---|---|---:|:---|
| `key` | `any` | --- | a key to be checked |

<a name="TNamedItemsCollection+get"></a>
##### **get(key)** => `any`

This method returns a value of a key given to the method. If the key not found, `null` is returned.

| parameter name | value type | default value | description |
|:---|---|---:|:---|
| `key` | `any` | --- | some key |

<a name="TNamedItemsCollection+add"></a>
##### **add(key)** => `boolean`

This method adds the given `key` to a collection members. If succeed, `true` returned.

| parameter name | value type | default value | description |
|:---|---|---:|:---|
| `key` | `any` | --- | some key |

> Note: if a given `key` is already exists, the method behavior depends on the value of a `forceIfKeyExists` collection settings option. In case, this option sets to `true` the existing key will be reset, if sets to `false` (*the default settings*) the method failed.

<a name="TNamedItemsCollection+set"></a>
##### **set(key, value)** => `boolean`

This method sets a value of a key addressed by the `key` parameter, if such a key exists. If succeed, `true` returned.

| parameter name | value type | default value | description |
|:---|---|---:|:---|
| `key` | `any` | --- | some key |
| `value` | `any` | --- | some value |

> Note: You can\'t set a key if a given `item` value is `undefined`. In such a case the method will failed.

<a name="TNamedItemsCollection+insert"></a>
##### **insert(key, value)** => `boolean`

This method sets a value of a key addressed by the `key` parameter. If succeed, `true` returned.

| parameter name | value type | default value | description |
|:---|---|---:|:---|
| `key` | `any` | --- | some key |
| `value` | `any` | --- | some value |

> Note: You can\'t set a key if a given `item` value is `undefined`. In such a case the method will failed.

<a name="TNamedItemsCollection+delete"></a>
##### **delete(key)** => `boolean`

This method deletes element addressed by a given `key` from a collection members. If succeed, `true` returned.

| parameter name | value type | default value | description |
|:---|---|---:|:---|
| `key` | `any` | --- | some key to delete |

<a name="TNamedItemsCollection+rename"></a>
##### **rename(key, value)** => `boolean`

This method renames a key addressed by the `key` parameter to a new name given by the `value`, if such a key exists.

| parameter name | value type | default value | description |
|:---|---|---:|:---|
| `key` | `any` | --- | some key |
| `value` | `any` | --- | some new key |

> Note: if a key addressed by a `value` already exists, the method behavior depends on the value of a `forceIfKeyExists` collection settings option. In case, this option sets to `true` the existing key will be replaced, if sets to `false` (*the default settings*) the method failed.

#### class methods (*special*)

<a name="TNamedItemsCollection+isValidKey"></a>
##### **isValidKey(value)** => `boolean`

This special method verifies if a value given by a `value` parameter is valid.

| parameter name | value type | default value | description |
|:---|---|---:|:---|
| `value` | `any` | --- | a key to be verified |

> Note: A value which returned by the method depends on the wrapped function. By default the `isValidKey` will use a static class method of the same name.

#### class methods (*static*)

<a name="TNamedItemsCollection.isValidKey"></a>
##### **isValidKey(value)** => `boolean`

This method verifies that a value given by a `value` parameter is valid according to the following base rules: the value is not equal to `undefined` or `null`, if so `true` is returned.

| parameter name | value type | default value | description |
|:---|---|---:|:---|
| `value` | `any` | --- | a key to be verified |

<a name="TNamedSetsCollection"></a>
### **TNamedSetsCollection**

This class extends a ['TNamedItemsCollection'](#TNamedItemsCollection) class with a new functionalities allow to handle a sets of elements.

#### class constructor

The class constructor creates a new instance of the class.

##### constructor parameters

The class receives an argument which contains an options listed below:

| option name | option type | default value | description |
|:---|---|---:|:---|
| `forceIfKeyExists` | `boolean` | `false` | alters behavior of the `add` and `rename` class methods. If set to `true`, the replacement of existing keys is allowed |
| `forceIfItemExists` | `boolean` | `false` | *reserved* |
| `userFn` | `object` | --- | a set of a user defined functions |

The `userFn` provides the set of next parameters:

| option name | option type | default value | description |
|:---|---|---:|:---|
| `checkIndexFn` | `function` | --- | defines a function that checks whether or not the given key can be accepted |
| `readIndexFn` | `function` | --- | defines a function that handles the given key |
| `checkItemFn` | `function` | --- | defines a function that checks whether or not the given item can be accepted |
| `readItemFn` | `function` | --- | defines a function that handles the given item |

The `checkIndexFn` function gets the key that must be verified and returns a `boolean` value as a result of a verification process.

The `readIndexFn` function gets a key and returns it transformed according to the rules set by some user logics.

The `checkItemFn` function gets the item that must be verified and returns a `boolean` value as a result of a verification process.

The `readItemFn` function gets an item and returns it transformed according to the rules set by some user logics.

#### class properties

<a name="TNamedSetsCollection+keys"></a>
##### **keys**

| property type | read only | description |
|---|---|:---|
| `any[]` | yes | returns an array of all collection keys |

<a name="TNamedSetsCollection+values"></a>
##### **values**

| property type | read only | description |
|---|---|:---|
| `any[]` | yes | returns an array of all collection values |

<a name="TNamedSetsCollection+entries"></a>
##### **entries**

| property type | read only | description |
|---|---|:---|
| `any[][]` | yes | returns an array of all collection entries as a "key-value" pairs |

<a name="TNamedSetsCollection+size"></a>
##### **size**

| property type | read only | description |
|---|---|:---|
| `number` | yes | returns a collection size |

#### class methods

<a name="TNamedSetsCollection+clear"></a>
##### **clear(\[key_1, \[key_2, \[key_N]]])** => `void`

This method deletes all items and all keys that collection holds in case if none was given. If any keys are given, then only the items belonging to the sets addressed by a listed keys will be deleted from the collection.

| parameter name | value type | default value | description |
|:---|---|---:|:---|
| `key_1`...`key_N` | `any` | --- | a key to removed |

<a name="TNamedSetsCollection+has"></a>
##### **has(key)** => `boolean`

This method checks if the given `key` exists in the collection.

| parameter name | value type | default value | description |
|:---|---|---:|:---|
| `key` | `any` | --- | a key to be checked |

<a name="TNamedSetsCollection+get"></a>
##### **get(key)** => `?Set`

This method returns a value of a key given to the method. If the key not found, `null` is returned.

| parameter name | value type | default value | description |
|:---|---|---:|:---|
| `key` | `any` | --- | some key |

<a name="TNamedSetsCollection+add"></a>
##### **add(key)** => `boolean`

This method adds the given `key` to a collection members. If succeed, `true` returned.

| parameter name | value type | default value | description |
|:---|---|---:|:---|
| `key` | `any` | --- | some key |

> Note: if a given `key` is already exists, the method behavior depends on the value of a `forceIfKeyExists` collection settings option. In case, this option sets to `true` the existing key will be reset, if sets to `false` (*the default settings*) the method failed.

<a name="TNamedSetsCollection+addTo"></a>
##### **addTo(key, value)** => `boolean`

This method adds a given item to a members of the set addressed by a given key. If succeed `true` is returned.

| parameter name | value type | default value | description |
|:---|---|---:|:---|
| `key` | `any` | --- | some key |
| `value` | `any` | --- | some value |

<a name="TNamedSetsCollection+set"></a>
##### **set(key, value)** => `boolean`

This method sets a value of a key addressed by the `key` parameter, if such a key exists. If succeed, `true` returned.

| parameter name | value type | default value | description |
|:---|---|---:|:---|
| `key` | `any` | --- | some key |
| `value` | `any` | --- | some value |

The `value` can be any valid item accepted by the instance or an array or a set of those items.

> Note: If 'value' is omitted the empty set is used by this methods.

<a name="TNamedSetsCollection+insert"></a>
##### **insert(key, value)** => `boolean`

This method sets a value of a key addressed by the `key` parameter. If succeed, `true` returned.

| parameter name | value type | default value | description |
|:---|---|---:|:---|
| `key` | `any` | --- | some key |
| `value` | `any` | --- | some value |

The `value` can be any valid item accepted by the instance or an array or a set of those items.

> Note: If 'value' is omitted the empty set is used by this methods.

<a name="TNamedSetsCollection+delete"></a>
##### **delete(key)** => `boolean`

This method deletes element addressed by a given `key` from a collection members. If succeed, `true` returned.

| parameter name | value type | default value | description |
|:---|---|---:|:---|
| `key` | `any` | --- | some key to delete |

<a name="TNamedSetsCollection+deleteFrom"></a>
##### **deleteFrom(key, value)** => `boolean`

This method deletes a given item from the set members addressed by a `key` parameter. If succeed `true` returned.

| parameter name | value type | default value | description |
|:---|---|---:|:---|
| `key` | `any` | --- | some key |
| `value` | `any` | --- | some value |

<a name="TNamedSetsCollection+rename"></a>
##### **rename(key, value)** => `boolean`

This method renames a key addressed by the `key` parameter to a new name given by the `value`, if such a key exists.

| parameter name | value type | default value | description |
|:---|---|---:|:---|
| `key` | `any` | --- | some key |
| `value` | `any` | --- | some new key |

> Note: if a key addressed by a `value` already exists, the method behavior depends on the value of a `forceIfKeyExists` collection settings option. In case, this option sets to `true` the existing key will be replaced, if sets to `false` (*the default settings*) the method failed.

<a name="TNamedSetsCollection+merge"></a>
##### **merge(key, src_1, \[src_2, \[src_N]])** => `boolean`

This method merge all sets addressed by a given arguments and places a resulted set into a `key`. All keys except a target will be deleted from the collection. If succeed `true` is returned.

| parameter name | value type | default value | description |
|:---|---|---:|:---|
| `key` | `any` | --- | some key |
| `src_1`...`src_N` | `any` | --- | some extra key |

<a name="TNamedSetsCollection+replace"></a>
##### **replace(key, oldItem, newItem)** => `boolean`

This method tries to replace an item with a new one in a set addressed by a given `key`. If succeed `true` returned.

| parameter name | value type | default value | description |
|:---|---|---:|:---|
| `key` | `any` | --- | some key |
| `oldItem` | `any` | --- | some element |
| `newItem` | `any` | --- | some element |

#### class methods (*special*)

<a name="TNamedSetsCollection+isValidKey"></a>
##### **isValidKey(value)** => `boolean`

This special method verifies if a value given by a `value` parameter is valid.

| parameter name | value type | default value | description |
|:---|---|---:|:---|
| `value` | `any` | --- | a key to be verified |

> Note: A value which returned by the method depends on the wrapped function. By default the `isValidKey` will use a static class method of the same name.

<a name="TNamedSetsCollection+isValidItem"></a>
##### **isValidItem(value)** => `boolean`

This special method verifies if a value given by an `item` parameter is valid.

| parameter name | value type | default value | description |
|:---|---|---:|:---|
| `value` | `any` | --- | an element to be verified |

> Note: A value which returned by the method depends on the wrapped function. By default the `isValidItem` will use a static class method of the same name.

<a name="TNamedSetsCollection+__createNewSet"></a>
##### **__createNewSet(\[arg_1, \[arg_2, \[arg_N]]])** => `Set`

This method creates a new set from a given arguments.

| parameter name | value type | default value | description |
|:---|---|---:|:---|
| `arg_1`...`arg_N` | `any` | --- | some element |

#### class methods (*static*)

<a name="TNamedSetsCollection.isValidKey"></a>
##### **isValidKey(value)** => `boolean`

This method verifies that a value given by a `value` parameter is valid according to the following base rules: the value is not equal to `undefined` or `null`, if so `true` is returned.

| parameter name | value type | default value | description |
|:---|---|---:|:---|
| `value` | `any` | --- | a key to be verified |

<a name="TNamedSetsCollection.isValidItem"></a>
##### **isValidItem(value)** => `boolean`

This method verifies that a value given by a `value` parameter is valid according to a followed base rules: the value is not equal to `undefined` or `null`, if so `true` is returned.

| parameter name | value type | default value | description |
|:---|---|---:|:---|
| `value` | `any` | --- | an element to be verified |

<a name="TItemsICollection"></a>
### **TItemsICollection**

This class implements an indexed collection of a named elements.

#### class constructor

The class constructor creates a new instance of the class.

##### constructor parameters

The class receives an arguments which contains an options listed bellow:

| option name | option type | default value | description |
|:---|---|---:|:---|
| `forceIfKeyExists` | `boolean` | `false` | alters behavior of the `add` and `rename` class methods. If set to `true`, the replacement of existing keys is allowed |
| `forceIfItemExists` | `boolean` | `false` | *reserved* |
| `userFn` | `object` | --- | a set of a user defined functions |

The `userFn` provides the set of next parameters:

| option name | option type | default value | description |
|:---|---|---:|:---|
| `checkIndexFn` | `function` | --- | defines a function that checks whether or not the given key can be accepted |
| `readIndexFn` | `function` | --- | defines a function that handles the given key |
| `checkItemFn` | `function` | --- | defines a function that checks whether or not the given item can be accepted |
| `readItemFn` | `function` | --- | defines a function that handles the given item |

The `checkIndexFn` function gets the key that must be verified and returns a `boolean` value as a result of a verification process.

The `readIndexFn` function gets a key and returns it transformed according to the rules set by some user logics.

The `checkItemFn` function gets the item that must be verified and returns a `boolean` value as a result of a verification process.

The `readItemFn` function gets an item and returns it transformed according to the rules set by some user logics.

#### class properties

<a name="TItemsICollection+ItemNames"></a>
##### **ItemNames**

| property type | read only | description |
|---|---|:---|
| `any[]` | yes | returns a list of an item names for all items that collection holds as an array |

<a name="TItemsICollection+CategoryNames"></a>
##### **CategoryNames**

| property type | read only | description |
|---|---|:---|
| `any[]` | yes | returns a list of all items categories existing within collection as an array |

#### class methods

<a name="TItemsICollection+clear"></a>
##### **clear()** => `void`

This method deletes a collection content.

<a name="TItemsICollection+hasItem"></a>
##### **hasItem(name\[, cat])** => `boolean`

This method checks if the item addressed by a given `name` exists in the collection.

| parameter name | value type | default value | description |
|:---|---|---:|:---|
| `name` | `any` | --- | an element key to be checked |
| `cat` | `any` | --- | an category key to be checked |

> \[!] NOTE: If a `cat` is given the method checks whether an element belongs to a target category.

<a name="TItemsICollection+getItem"></a>
##### **getItem(name)** => `?any`

This method returns the item addressed by a given `name` if it exists in the collection. If not `null` is returned.

| parameter name | value type | default value | description |
|:---|---|---:|:---|
| `name` | `any` | --- | an element key |

<a name="TItemsICollection+getItemCategories"></a>
##### **getItemCategories(name)** => `any[]`

This method returns a list of all categories for the element addressed by a given `name` if it exists in the collection.

| parameter name | value type | default value | description |
|:---|---|---:|:---|
| `name` | `any` | --- | an element key |

<a name="TItemsICollection+addItem"></a>
##### **addItem(name, item\[, cat_1\[, cat_2\[, cat_N]]])** => `boolean`

This method adds an element with a given name to the collection. If a categories listed by an arguments an element becomes its member. If succeed `true` is returned.

| parameter name | value type | default value | description |
|:---|---|---:|:---|
| `name` | `any` | --- | an element key |
| `item` | `any` | --- | an element |
| `cat_1`...`cat_N` | `any` | --- | a category key |

<a name="TItemsICollection+addItemToCategory"></a>
##### **addItemToCategory(name, cat_1\[, cat_2\[, cat_N]])** => `boolean`

This method adds the item addressed by a `name` parameter to the members of all categories listed by an arguments. If succeed `true` is returned.

| parameter name | value type | default value | description |
|:---|---|---:|:---|
| `name` | `any` | --- | an element key |
| `cat_1`...`cat_N` | `any` | --- | a category key |

<a name="TItemsICollection+delItem"></a>
##### **delItem(name)** => `boolean`

This method deletes the item addressed by a given `name` from the collection. If succeed `true` is returned.

| parameter name | value type | default value | description |
|:---|---|---:|:---|
| `name` | `any` | --- | an element key |

<a name="TItemsICollection+delItemFromCategory"></a>
##### **delItemFromCategory(name, cat_1\[, cat_2\[, cat_N]])** => `boolean`

This method deletes the item addressed by a `name` parameter from the members of all categories listed by an arguments. If succeed `true` is returned.

| parameter name | value type | default value | description |
|:---|---|---:|:---|
| `name` | `any` | --- | an element key |
| `cat_1`...`cat_N` | `any` | --- | a category key |

<a name="TItemsICollection+renameItem"></a>
##### **renameItem(name, value)** => `boolean`

This method renames a key addressed by the `name` parameter to a new name given by the `value`, if such a key exists.

| parameter name | value type | default value | description |
|:---|---|---:|:---|
| `name` | `any` | --- | some key |
| `value` | `any` | --- | some new key |

> Note: if a key addressed by a `value` already exists, the method behavior depends on the value of a `forceIfKeyExists` collection settings option. In case, this option sets to `true` the existing key will be replaced, if sets to `false` (*the default settings*) the method failed.

<a name="TItemsICollection+replaceItem"></a>
##### **replaceItem(name, item)** => `boolean`

This method replaces the item addressed by a `name` with a new `item`. If succeed `true` is returned.

| parameter name | value type | default value | description |
|:---|---|---:|:---|
| `name` | `any` | --- | some key |
| `item` | `any` | --- | some new element |

<a name="TItemsICollection+hasCategory"></a>
##### **hasCategory(name)** => `boolean`

This method checks if the category addressed by a given `name` exists in the collection.

| parameter name | value type | default value | description |
|:---|---|---:|:---|
| `name` | `any` | --- | some category key |

<a name="TItemsICollection+getCategory"></a>
##### **getCategory(name)** => `?Set`

This method returns the items that are belongs to the category addressed by a given name if the targeted category exists in the collection. If not `null` is returned.

| parameter name | value type | default value | description |
|:---|---|---:|:---|
| `name` | `any` | --- | some category key |

<a name="TItemsICollection+addCategory"></a>
##### **addCategory(name)** => `boolean`

This method adds the category named by a `name` parameter in the collection. If succeed `true` is returned.

| parameter name | value type | default value | description |
|:---|---|---:|:---|
| `name` | `any` | --- | some category key |

<a name="TItemsICollection+delCategory"></a>
##### **delCategory(name)** => `boolean`

This method deletes the category named by a `name` parameter from the collection. If succeed `true` is returned.

| parameter name | value type | default value | description |
|:---|---|---:|:---|
| `name` | `any` | --- | some category key |

<a name="TItemsICollection+renameCategory"></a>
##### **renameCategory(name, value)** => `boolean`

This method renames the category addressed by a `name` to its `value`. If succeed `true` is returned.

| parameter name | value type | default value | description |
|:---|---|---:|:---|
| `name` | `any` | --- | some category key |
| `value` | `any` | --- | some new category key |

> Note: if a key addressed by a `value` already exists, the method behavior depends on the value of a `forceIfKeyExists` collection settings option. In case, this option sets to `true` the existing key will be replaced, if sets to `false` (*the default settings*) the method failed.

<a name="TItemsICollection+mergeCategories"></a>
##### **mergeCategories(name, src_1, \[src_2, \[src_N]])** => `boolean`

This method merge all sets addressed by a given arguments and places a resulted set into a `name`. All keys except a target will be deleted from the collection. If succeed `true` is returned.

| parameter name | value type | default value | description |
|:---|---|---:|:---|
| `name` | `any` | --- | some category key |
| `src_1`...`src_N` | `any` | --- | some extra key |
