|***rev.*:**|0.1.23|
|:---|---:|
|***date***:|2025-03-25|

## Introduction

This paper contains a descriptions of all helper functions presented in "`baseFunc.js`".

### Logical functions

<a name="isValidId"></a>
#### **isValidId(value\[, options])** => `boolean`

This function returns `true` if a `value` is a valid identifier.

| parameter name | value type | default value | description |
|:---|---|---:|:---|
| `value` | `any` | --- | some element to be verified |
| `options` | `boolean`, `object` | --- | some options |

For more details on an `options` parameter see the description for a [`valueToIDString`](#valueToIDString) function.

<a name="isValidIndex"></a>
#### **isValidIndex(index)** => `boolean`

This function returns `true` if the value given by the `index` parameter can be interpreted as a valid index value.

| parameter name | value type | default value | description |
|:---|---|---:|:---|
| `value` | `any` | --- | some element to be verified |

For more details see the description for a [`valueToIndex`](#valueToIndex) function.

<a name="isEmptyString"></a>
#### **isEmptyString(value)** => `boolean`

This function returns `true` if a given value represents an empty string.

| parameter name | value type | default value | description |
|:---|---|---:|:---|
| `value` | `any` | --- | some element to be verified |

> Note: Any string filled with a space characters only regarded by this function as an empty string.

<a name="isEmptyOrNotString"></a>
#### **isEmptyOrNotString(value)** => `boolean`

> since: \[v0.1.4]

This function returns `true` if a given value represents an empty string or is not a string at all.

| parameter name | value type | default value | description |
|:---|---|---:|:---|
| `value` | `any` | --- | some element to be verified |

> Note: Any string filled with a space characters only regarded by this function as an empty string.

<a name="isNotEmptyString"></a>
#### **isNotEmptyString(value)** => `boolean`

This function returns `true` if a given value represents a non-empty string.

| parameter name | value type | default value | description |
|:---|---|---:|:---|
| `value` | `any` | --- | some element to be verified |

> Note: Any string filled with a space characters only regarded by this function as an empty string.

<a name="isNullOrUndef"></a>
#### **isNullOrUndef(value)** => `boolean`

This function returns `true` if a given value is equal to `undefined` or `null`.

| parameter name | value type | default value | description |
|:---|---|---:|:---|
| `value` | `any` | --- | some element to be verified |

<a name="isInteger"></a>
#### **isInteger(value)** => `boolean`

This function returns `true` if a given value is an integer number.

| parameter name | value type | default value | description |
|:---|---|---:|:---|
| `value` | `any` | --- | some element to be verified |

> Note: technically `isInteger` is simply a shorter reference to a `Number.isInteger` method.

<a name="inRangeNum"></a>
#### **inRangeNum(min, max, value)** => `boolean`

This function returns `true` if a given `value` is in range between `min` and `max` values or equal to one of them.

| parameter name | value type | default value | description |
|:---|---|---:|:---|
| `min` | `number` | --- | minimum value |
| `max` | `number` | --- | maximum value |
| `value` | `number` | --- | some value to check |

### Special functions

<a name="tryNotNullish"></a>
#### **tryNotNullish(arg_1\[, arg_2\[, arg_N]])** => `?any`

This function scans a list of a given arguments and returns the first one that is not equal to `undefined` or `null`. If no one was found, `null` will be returned.

| parameter name | value type | default value | description |
|:---|---|---:|:---|
| `arg_1`...`arg_N` | `any` | --- | some value to check |

<a name="readAsBoolEx"></a>
#### **readAsBoolEx(value\[, defValue])** => `boolean`

This function similar to the [`readAsBool`](#readAsBool) function but in a contrast tries to convert a given value to a `boolean` type before the result is retuned.

| parameter name | value type | default value | description |
|:---|---|---:|:---|
| `value` | `any` | --- | some value to transform |
| `defValue` | `boolean` | --- | some default value to use |

The `value` conversion process consists of the following rules:

1. if `value` is equal to `null`, `false` is returned;
2. if `value` is a number then:
   - `false` will be returned if a value is equal to `0`,
   - `true` will be returned in other cases (except `NaN`),
   - in cases if `NaN` is matched a `defValue` is returned;
3. if `value` is a string then:
   - if a values are `null` or `false` (also its uppercase variants), `false` is returned,
   - if a values are `off` (also its uppercase variants), `false` is returned `[since v0.2.1]`,
   - if a values are `no`, `No` or `NO`, `false` is returned `[since v0.2.1]`,
   - if a value is `true` (also its uppercase variant), `true` is returned,
   - if a values are `on` (also its uppercase variants), `true` is returned `[since v0.2.1]`,
   - if a values are `yes`, `Yes` or `YES`, `true` is returned `[since v0.2.1]`,
   - for values that can represents a numbers the rule "2" is applied,
   - in other cases a `defValue` will be returned.

> Note: For a strings, the value is trimmed before it's processed.

<a name="readAsNumberEx"></a>
#### **readAsNumberEx(value\[, defValue])** => `number`

This function similar to the [`readAsNumber`](#readAsNumber) function but in contrast tries to convert a given value to a `number` type before the result is retuned.

| parameter name | value type | default value | description |
|:---|---|---:|:---|
| `value` | `any` | --- | some value to transform |
| `defValue` | `number` | --- | some default value to use |

The `value` conversion process consists of the following rules:

1. if `value` is equal to `null`, `0` is returned;
2. if `value` is a `boolean` then:
   - if `false`, `0` will be returned,
   - if `true`, `1` will be returned,
3. if `value` is a `number` and `NaN` is matched a `defValue` is returned;
4. if `value` is a string then:
   - if a values are `null` (also its uppercase variants), `0` is returned,
   - for values that can represents a booleans (i.e, `false` or `true` and also its uppercase variants) the rule "2" is applied,
   - for values that can represents a numbers the rule "3" is applied,
   - in other cases a `defValue` will be returned.

> Note: For a strings, the value is trimmed before it's processed.

### Other functions

<a name="valueToIndex"></a>
#### **valueToIndex(value)** => `number`

This function tries to convert a given value to a valid index value.

| parameter name | value type | default value | description |
|:---|---|---:|:---|
| `value` | `any` | --- | some value to transform |

> Note: For this case a "valid index value" is a positive integer or `0` or `-1` (*a later is a "special value", meanwhile not treated as an index itself. It used similar to a `NaN` for a `number` type*).

<a name="valueToIDString"></a>
#### **valueToIDString(value\[, options])** => `string`

> since: \[v0.1.3]

This function tries to convert a given value to a valid `ID_STRING` value. If function failed, `null` is returned.

| parameter name | value type | default value | description |
|:---|---|---:|:---|
| `value` | `any` | --- | some value |
| `options` | `boolean`, `object` | --- | some options |

An `options` parameter must be a `boolean` or an `object`.

> \[!] NOTE: `[since v0.2.3]` a use of a `boolean` type as a value of an `options` are deprecated and will be dropped soon.

If `options` is an `object` it contains the settings listed in table below:

| name option | option type | default value | description |
|:---|---|---:|:---|
| `onlyNES` | `boolean` | `true` | a flag that defines whether or not the empty value is allowed |
| `ignoreNumbers` | `boolean` | `false` | `[since v0.2.1]` a flag that defines whether or not a number can be treated as a valid ID |
| `ignoreCase` | `boolean` | `false` | `[since v0.2.2]` a flag that defines whether or not a character case must be ignored. All characters in the result will be transformed to a lower case |

If `options` is of a `boolean` type the function treats it as a value of `options.onlyNES`.

<a name="tryToBool"></a>
#### **tryToBool(value)** => `?boolean`

> since: \[v0.2.1]

This function tries to read a given value and returns it if that value has a `boolean` type. If function failed, `null` is returned.

| parameter name | value type | default value | description |
|:---|---|---:|:---|
| `value` | `any` | --- | some value |

> NOTE: For details of a `value` parameter see the description for the [`readAsBool`](#readAsBool) function.

<a name="tryToNumber"></a>
#### **tryToNumber(value)** => `?number`

> since: \[v0.2.1]

This function tries to read a given value and returns it if that value has a `number` type. If function failed, `null` is returned.

| parameter name | value type | default value | description |
|:---|---|---:|:---|
| `value` | `any` | --- | some value |

> NOTE: For `value` parameter the function accepts a strings if those can be converted to a number.

<a name="readAsBool"></a>
#### **readAsBool(value\[, defValue])** => `boolean`

This function reads a given value and returns it if that value has a `boolean` type.

| parameter name | value type | default value | description |
|:---|---|---:|:---|
| `value` | `any` | --- | some value to transform |
| `defValue` | `boolean` | --- | some default value to use |

If function failed, one of the following cases may occur:

- if `defValue` is defined and has a `boolean` type, then `defValue` will be returned;
- `false` will be returned in other cases.

> NOTE: `[since v0.1.3]` For `value` parameter the function accepts a strings if those value equal to `true` or `false` (*both strictly in low case and each may be padded with spaces before and/or after*) and treats those as booleans.

> NOTE: `[since v0.2.0]` For `value` parameter the function also accepts a strings if those value equal to `TRUE` or `FALSE` (*both strictly in upper case and each may be padded with spaces before and/or after*) and treats those as booleans.

<a name="readAsNumber"></a>
#### **readAsNumber(value\[, defValue])** => `number`

This function reads a given value and returns it if that value has a `number` type.

| parameter name | value type | default value | description |
|:---|---|---:|:---|
| `value` | `any` | --- | some value to transform |
| `defValue` | `number` | --- | some default value to use |

If function failed, one of the following cases may occur:

- if `defValue` is defined and has a `number` type, then `defValue` will be returned;
- `0` will be returned in other cases.

> NOTE: `[since v0.1.3]` For `value` parameter the function accepts a strings if those can be converted to a number.

<a name="readAsString"></a>
#### **readAsString(value\[, options])** => `string`

> since: \[v0.2.0]

This function reads a given value and returns it as a string.

| parameter name | value type | default value | description |
|:---|---|---:|:---|
| `value` | `any` | --- | some value to transform |
| `options` | `boolean`, `string` or `object` | --- | some options |

An `options` parameter must be a `boolean`, a `string` or an `object`.

> \[!] NOTE: `[since v0.2.2]` a use of a `boolean` or `string` types as a value of an `options` are deprecated and will be dropped soon.

If `options` is an `object` it contains the settings listed in table below:

| name option | option type | default value | description |
|:---|---|---:|:---|
| `useTrim` | `boolean` | `false` | defines whether or not the resulting string must to be trimmed |
| `letterCase` | `string` | --- | if given defines a function behavior on the resulting string |
| `numberToString` | `boolean` | `false` | if set a values of `number` types will be converted to a string |
| `boolToString` | `boolean` | `false` | if set a values of `boolean` types will be converted to a string |
| `boolToUpperCase` | `boolean` | `false` | if set a values of `boolean` types when converted to a string will be present in upper case |
| `defValue` | `string` | `EMPTY_STRING` | if given represents a value which will be returned in case the function failed |

If `options` is of a `boolean` type the function treats it as a value of `options.useTrim`.

If `options` is of a `string` type the function treats it as a value of `options.defValue`.

The function behavior for `options.letterCase` settings see in the table below:

| option value | description |
|:---|:---|
| `upper` | the resulting string must to be present in upper case |
| `lower` | the resulting string must to be present in lower case |
| any other values | the resulting string will be not transformed |

### Experimental functions

> Note: Purpose of those functions will be discussed and some may be deprecate and make obsolete or functionality may be altered. So use it with cautions in mind.

<a name="isNotEmptyStringEx"></a>
#### **isNotEmptyStringEx(arg_1\[, arg_2\[, arg_N]])** => `boolean`

This function reads a given arguments and returns `true` if none of them can be considered as an empty string value.

| parameter name | value type | default value | description |
|:---|---|---:|:---|
| `arg_1`...`arg_N` | `any` | --- | some value to check |
