|***rev.*:**|0.1.20|
|:---|---:|
|***date***:|2025-03-29|

## Introduction

This paper contains a descriptions of all object functions presented in "`baseObj.js`".

### Logical functions

<a name="isArray"></a>
#### **isArray(value)** => `boolean`

This function checks if a given value represents an `Array` and if so returns `true`.

| parameter name | value type | default value | description |
|:---|---|---:|:---|
| `value` | `any` | --- | some element to be verified |

> Note: technically `isArray` is simply a shorter reference to an `Array.isArray` method.

<a name="isObject"></a>
#### **isObject(value)** => `boolean`

This function checks if a given value represents an `Object` instance of any kind and if so returns `true`.

| parameter name | value type | default value | description |
|:---|---|---:|:---|
| `value` | `any` | --- | some element to be verified |

<a name="isPlainObject"></a>
#### **isPlainObject(value)** => `boolean`

This function checks if a given value represents a simple object and if so returns `true`.

| parameter name | value type | default value | description |
|:---|---|---:|:---|
| `value` | `any` | --- | some element to be verified |

> Note: technically for current implementation the method checks whether an object not present an instance of `Array`. So for example an instance of `Set` will pass a check process.

<a name="isDateObject"></a>
#### **isDateObject(value)** => `boolean`

This function checks if a given value represents a `DateTime` object and if so returns `true`.

| parameter name | value type | default value | description |
|:---|---|---:|:---|
| `value` | `any` | --- | some element to be verified |

<a name="isPropertyExists"></a>
#### **isPropertyExists(object, prop)** => `boolean`

This function checks if a given object has the property given by a `prop` parameter and if so returns `true`.

| parameter name | value type | default value | description |
|:---|---|---:|:---|
| `object` | `object` | --- | some instance |
| `prop` | `string` | --- | some name of the instance property |

> Note: This function will fail if a given name for the searched object attribute represents an object method. In such case you need to use [`isMethodExists`](#isMethodExists).

<a name="isMethodExists"></a>
#### **isMethodExists(object, method)** => `boolean`

This function checks if a given object has the method given by a `method` parameter and if so returns `true`.

| parameter name | value type | default value | description |
|:---|---|---:|:---|
| `object` | `object` | --- | some instance |
| `method` | `string` | --- | some name of the instance method |

> Note: This function will fail if a given name for the searched object attribute represents an object property. In such case you need to use [`isPropertyExists`](#isPropertyExists).

### Other functions

<a name="arrayFrom"></a>
#### **arrayFrom(value)** => `any[]`

This function transforms a given `value` to an array.

| parameter name | value type | default value | description |
|:---|---|---:|:---|
| `value` | `any` | --- | some element to transform |

> Note: technically `arrayFrom` is simply a shorter reference to an `Array.from` method.

<a name="valueToArray"></a>
#### **valueToArray(value)** => `any[]`

This function transforms a given `value` to an array in case it is not.

| parameter name | value type | default value | description |
|:---|---|---:|:---|
| `value` | `any` | --- | some element to transform |

>\[!] NOTE:
>- If given value is `undefined` or `null` an empty array will be returned.
>- If given value is `Set` or `Map` it will be transformed to an array.

<a name="valueToEntry"></a>
#### **valueToEntry(value)** => `any[]`

> since: \[v0.2.1]

This function transforms a given `value` to an entry which represents a pair of \<key> and its value. If failed `null` is returned.

| parameter name | value type | default value | description |
|:---|---|---:|:---|
| `value` | `any` | --- | some element to transform |

<a name="valueToEntries"></a>
#### **valueToEntries(value\[, options])** => `any[][]`

> since: \[v0.1.4]

This function transforms a given `value` to an array of an entries which represents a pairs of \<key> and its value.

| parameter name | value type | default value | description |
|:---|---|---:|:---|
| `value` | `any` | --- | some element to transform |
| `options` | `any` | --- | some options |

For details on `options` parameter see a description of a [`readAsString`](baseFunc.md) function.

<a name="readAsListE"></a>
#### **readAsListE(options, arg_1\[, arg_2\[, arg_N]])** => `any[]`

This function reads its arguments and returns an array of an elements according to the given options.

| parameter name | value type | default value | description |
|:---|---|---:|:---|
| `options` | `boolean`, `object` | --- | some options |
| `arg_1`...`arg_N` | `any` | --- | some element to transform |

The `options` parameter must be a `boolean` or an `object`.

> \[!] NOTE: `[since v0.2.3]` a use of a `boolean` type as a value of an `options` are deprecated and will be dropped soon.

If `options` is of an `object` type it contains the following parameters:

| option name | value type | default value | description |
|:---|---|---:|:---|
| `keepEmpty` | `boolean` | `false` | `[since v0.1.4]` defines a behavior of a `checkItemFn` function |
| `unfoldArray` | `boolean` | `false` | `[since v0.1.4]` if set to `true` require to unfold 1st level of arrays listed in `args` (*also applied to a set instances if present*) |
| `userFn` | `object` | --- | `[since v0.2.0]` contains a user defined callbacks to perform ops upon a given elements |
| `userOptions` | `any` | --- | `[since v0.2.0]` <*reserved*> |

If an `options` is of a `boolean` type the function treats it as a value of `options.keepEmpty`.

The `options.userFn` contains the following parameters:

| option name | value type | default value | description |
|:---|---|---:|:---|
| `checkItemFn` | `function` | --- | defines a user function that checks whether or not the given element can be accepted |
| `readItemFn` | `function`| --- | defines a user function that handles the given element |

The `checkItemFn` function gets two parameters:

1. `item` - the element that must be verified;
2. `keepEmpty` - a `boolean` value which defined whether or not an element that failed a verification process must be accepted.

The `readItemFn` function gets an element and returns it transformed according to the rules set by some user logics.

<a name="readAsListS"></a>
#### **readAsListS(options, arg_1\[, arg_2\[, arg_N]])** => `string[]`

This function reads its arguments and returns an array of strings according to the given options.

| parameter name | value type | default value | description |
|:---|---|---:|:---|
| `options` | `boolean`, `string`, `object` | --- | some options |
| `arg_1`...`arg_N` | `any` | --- | some element to transform |

An `options` parameter must be a `boolean`, a `string` or an `object`.

> \[!] NOTE: `[since v0.2.2]` a use of a `boolean` or `string` types as a value of an `options` are deprecated and will be dropped soon.

If an `options` is an `object` it contains the settings listed in table below:

| option name | value type | default value | description |
|:---|---|---:|:---|
| `keepEmpty` | `boolean` | `false` | `[since v0.1.3]` defines whether or not the empty string must to be kept |
| `unfoldArray` | `boolean` | `false` | `[since v0.1.3]` defines whether or not the arrays must be unwrapped |
| `unfoldLevel` | `number` | `1` | `[since v0.1.3]` defines a level at which the arrays must be unwrapped |
| `userOptions` | `any` | --- | `[since v0.2.2]` a user defined options |

If `options` is of a `boolean` type the function treats it as a value of `options.userOptions.useTrim`.

If `options` is of a `string` type the function treats it as a default value.

The `options.unfoldLevel` applied only if an `option.unfoldArray` is set to `true`. It must be equal or greater then `0` if else it sets to default value. Also a value of `-1` is accepted, in such a case no limit will be applied.

Also `options.userOptions` parameter may contain other settings. For those see a description to an `options` parameter of a [`readAsString`](baseFunc.md#readAsString) function

### Experimental functions

> Note: Purpose of those functions will be discussed and some may be deprecate and make obsolete or functionality may be altered. So use it with cautions in mind.

<a name="doMethodIfExists"></a>
#### **doMethodIfExists(object, method, arg_1\[, arg_2\[, arg_N])** => `any`

This function tries to call method named by a `method` parameter for a given object if such a method exists.

| parameter name | value type | default value | description |
|:---|---|---:|:---|
| `object` | `object` | --- | some instance |
| `method` | `string` | --- | some name of the instance method |
| `arg_1`...`arg_N` | `any` | --- | some arguments to pass to the instance method |

<a name="valueToArrayEx"></a>
#### **valueToArrayEx(value\[, options])** => `any[]`

> since: \[v0.1.1]

This function transforms a given `value` to an array in case it is not. Unlike `valueToArray` if given value is `null` this function returns an array containing only one element equal to `null`.

| parameter name | value type | default value | description |
|:---|---|---:|:---|
| `value` | `any` | --- | some element to transform |
| `callback` | `function`, `object`| --- | defines a user function that handles transformation or `[since v0.2.3]` some options |

The function can receive a "callback"-function which purpose is verify and transform elements through a conversion process. The "callback"-function if given is called on every element of a processed array, receives only two arguments (`element` and `index`) and must return that element after its transformation.

The `options` parameter must be a `function` or an `object`.

If `options` is of an `object` type it contains the following parameters:

| option name | value type | default value | description |
|:---|---|---:|:---|
| `mapFn` | `function` | --- | defines some user procedure to transform an array elements |
| `treatMapsAsObject` | `boolean` | `false` | defines whether to treat a `Map`-object as an object or like an array |
