// [v0.1.087-20250329]

// === module init block ===

const {
  valueToIndex,
} = require('../baseFunc.js');

const {
  isPlainObject,
  valueToArray,
} = require('../baseObj.js');

// === module extra block (helper functions) ===

// === module main block ===

/***
 * (* constant definitions *)
 */

/***
 * (* function definitions *)
 */

/***
 * (* class definitions *)
 */

/**
* @callback cb_forEachProcEx
* @param {any} item
* @param {number} [index]
* @param {any[]} [arr]
* @returns {void}
* @description user defined procedure to process an array elements
*/

/**
 * @classdesc This class implements an interface of the items list
 */
class TItemsList {
  /** @property {any[]} */
  #_items;
  /** @property {number} */
  #_curIndex;

  /**
   * @description Creates an instance of the items list
   */
  constructor() {
    this.#_items = [];
    this.#_curIndex = -1;
  }

  /**
   * @property {number}
   * @readonly
   * @description Contains qty of the list elements
   */
  get count() {
    return this.#_items.length;
  }

  /**
   * @property {number}
   * @description Contains an index of the current element
   */
  get curIndex() {
    return this.#_curIndex;
  }

  set curIndex(index) {
    this.setIndex(index);
  }

  /**
   * @property {?any}
   * @description Points to an element addressed by a current index
   */
  get curItem() {
    return this.getItem(this.curIndex);
  }

  set curItem(item) {
    this.setItem(this.curIndex, item);
  }

  /**
   * @since 0.2.0
   * @property {number}
   * @readonly
   * @description Returns a minimum index in the list
   */
  get minIndex() {
    return this.count !== 0 ? 0 : -1;
  }

  /**
   * @since 0.2.0
   * @property {number}
   * @readonly
   * @description Returns a maximum index in the list
   */
  get maxIndex() {
    return this.count - 1;
  }

  /**
   * @returns {void}
   * @description Deletes all items from the list.
   */
  clear() {
    this.#_items = [];
    this.#_curIndex = -1;
  }

  /**
   * @returns {boolean}
   * @description Indicates whether a list is empty.
   */
  isEmpty() {
    return this.count === 0;
  }

  /**
   * @returns {boolean}
   * @description Indicates whether a list is not empty.
   */
  isNotEmpty() {
    return this.count > 0;
  }

  /**
   * @param {(number|string)} value - value to check
   * @returns {boolean}
   * @description Checks whether a given value is a valid index value and
   *              it is not exceeds an index range within the list.
   */
  chkIndex(value) {
    const index = valueToIndex(value);
    return index !== -1 && index < this.count;
  }

  /**
   * @param {(number|string)} value - value of an index to be set
   * @returns {boolean}
   * @description Sets a current index in the list.
   */
  setIndex(value) {
    if (!this.chkIndex(value)) return false;
    this.#_curIndex = Number(value);
    return true;
  }

  /**
   * @returns {void}
   * @description Resets a current list index.
   */
  rstIndex() {
    this.#_curIndex = -1;
  }

  /**
   * @param {any} item - item to search
   * @param {(number|string)} [index=0] - a position to start
   * @returns {number}
   * @description Searchs an index of a given item in the list and
   *              returns its index.
   */
  srchIndex(item, index = 0) {
    const sPos = valueToIndex(index);
    return this.#_items.indexOf(item, sPos !== -1 ? sPos : 0);
  }

  /**
   * @param {(number|string)} index - index of the element
   * @returns {any}
   * @description Returns an item by a given index.
   */
  getItem(index) {
    if (this.chkIndex(index)) return this.#_items[Number(index)];
  }

  /**
   * @param {(number|string)} index - index of the element
   * @param {?any} item - an element to be set
   * @returns {boolean}
   * @description Places an item into a position addressed by a given index.
   */
  setItem(index, item) {
    if (item === undefined || !this.chkIndex(index)) return false;
    this.#_items[Number(index)] = item;
    return true;
  }

  /**
   * @param {any} item - element to be added
   * @returns {boolean}
   * @description Adds an item to a list.
   */
  addItem(item) {
    if (item === undefined) return false;
    this.#_items.push(item);
    return true;
  }

  /**
   * @param {any} item - element to be added
   * @param {boolean} [opt=false] - indicates whether to correct a current index
   * @returns {number}
   * @description Adds an item to a list.
   */
  addItemEx(item, opt = false) {
    let index = -1;
    if (item !== undefined) {
      const forceCI = typeof opt === 'boolean' ? opt : false;
      index = this.count;
      this.#_items.push(item);
      if (forceCI) this.#_curIndex = index;
    };
    return index;
  }

  /**
   * @param {(number|string)} index - index of the element
   * @param {boolean} [opt=true] - indicates whether to correct a current index
   * @returns {boolean}
   * @see TItemsList.delItemEx
   * @description Deletes an item from a list.
   */
  delItem(...args) {
    return this.delItemEx(...args) !== undefined;
  }

  /**
   * @param {(number|string)} index - index of the element
   * @param {boolean} [opt=true] - indicates whether to correct a current index
   * @returns {any}
   * @description Deletes an item from a list.
   */
  delItemEx(index, opt = true) {
    if (!this.chkIndex(index)) return;
    const cPos = Number(index);
    const item = this.#_items.splice(cPos, 1)[0];
    let current = this.curIndex;
    if (current !== -1) {
      const forceCI = typeof opt === 'boolean' ? opt : true;
      const count = this.count;
      if (forceCI) {
          if (cPos < current) {
            this.#_curIndex = --current;
          } else {
            if (current === count) this.#_curIndex = --current;
          };
      } else {
        if (current === count) this.#_curIndex = -1;
      };
    };
    return item;
  }

  /**
   * @typedef {Object} OPT_loaditems
   * @property {boolean} [useClear=true] - indicates whether to clear the list before load
   * @property {boolean} [resetIndex=false] - indicates whether reset current index
   * @property {(number|string)} [curIndex] - defines a value for a current index to be set
   * @description A settings for the list load ops
   */

  /**
   * @param {any[]} list - list of an elements to load
   * @param {(boolean|OPT_loaditems)} [opt] - options
   * @returns {number}
   * @description Adds an items to a list.
   * @todo [since `v0.2.2`] `opt` as `boolean` deprecated
   */
  loadItems(list, opt) {
    const items = valueToArray(list);
    let result = 0;
    if (items.length > 0) {
      /** @type {OPT_loaditems} */
      let {
        useClear,
        resetIndex,
        curIndex,
      } = isPlainObject(opt) ? opt : { useClear: opt };
      if (typeof useClear !== 'boolean') useClear = true;
      if (typeof resetIndex !== 'boolean') resetIndex = false;
      if (useClear) {
        this.clear();
        resetIndex = false;
      };
      const count = this.count;
      items.forEach((item) => { this.addItem(item); });
      result = this.count - count;
      if (!this.setIndex(curIndex) && resetIndex) this.rstIndex();
    };
    return result;
  }

  /**
   * @since 0.2.1
   * @param {cb_forEachProcEx} cb - a callback function
   * @returns {void}
   * @description Calls a given function for each list element
   */
  forEach(cb) {
    if (typeof cb === 'function') this.#_items.forEach(cb);
  }

};

/**
 * @augments TItemsList
 * @classdesc This class enhanced a capabilities implemented
 *            in the `TItemsList` class
 */
class TItemsListEx extends TItemsList {
  /** @property {number} */
  #_lastSrchIndex;

  /**
   * @description Creates an instance of `TItemsListEx` class
   */
  constructor() {
    super();
    this.#_lastSrchIndex = -1;
  }

  /**
   * @property {number}
   * @readonly
   * @description Returns a value of a previous index
   */
  get prevIndex() {
    let result = super.curIndex;
    if (result > 0) result--;
    return result;
  }

  /**
   * @property {number}
   * @readonly
   * @description Returns a value of a next index
   */
  get nextIndex() {
    let result = this.maxIndex;
    let maxIndex = result;
    if (result !== -1) {
      result = super.curIndex;
      if (result !== -1 && result < maxIndex) result++;
    };
    return result;
  }

  /**
   * @returns {void}
   * @description Deletes all items from the list.
   */
  clear() {
    super.clear();
    this.#_lastSrchIndex = -1;
  }

  /**
   * @param {any} value - value to check
   * @param {boolean} [opt=false] - defines a type of result
   * @returns {(boolean|number)}
   * @description Checks whether a given value is a valid index value and
   *              it is not exceeds an index range within the list.
   *              The type of result returned by a method defined by
   *              an `opt` parameter.
   */
  chkIndexEx(value, opt = false) {
    const index = valueToIndex(value);
    const result = index !== -1 && index < super.count;
    if (typeof opt === 'boolean' ? opt : false) {
      return result ? index : -1;
    } else {
      return result;
    };
  }

  /**
   * @param {any} item - item to search
   * @param {(number|string)} [index=0] - a position to start
   * @returns {number}
   * @see TItemsList.srchIndex
   * @description Searchs an index of a given item in the list and
   *              returns its index.
   */
  srchIndex(...args) {
    const index = super.srchIndex(...args);
    this.#_lastSrchIndex = index;
    return index;
  }

  /**
   * @param {any} item - item to search
   * @returns {number}
   * @description Searchs an index of a given item in the list and
   *              returns its index. The search starts from a last
   *              position of a previous call.
   */
  srchNextIndex(item) {
    let index = this.#_lastSrchIndex;
    if (index !== -1) {
      if (index < this.maxIndex) {
        index = this.srchIndex(item, index + 1);
      } else {
        index = -1;
      }
      this.#_lastSrchIndex = index;
    };
    return index;
  }

};

// === module exports block ===

module.exports.TItemsList = TItemsList;
module.exports.TItemsListEx = TItemsListEx;
